/// <reference path="../native.ts" />
/// <reference path="../object.ts" />

namespace trusted.cms {
    /**
     * Collection of CertInfo
     *
     * @export
     * @class CertInfoCollection
     * @extends {BaseObject<native.CMS.CertInfoCollection>}
     * @implements {Collection.ICollection}
     */
    export class CertInfoCollection extends BaseObject<native.CMS.CertInfoCollection> implements core.ICollection {

        /**
         * Creates an instance of CertInfoCollection.
         *
         * @param {native.CMS.CertInfoCollection} nativeHandle
         *
         * @memberOf CertInfoCollection
         */
        constructor(nativeHandle: native.CMS.CertInfoCollection) {
            super();

            this.handle = nativeHandle;
        }

        /**
         * Return element by index from collection
         *
         * @param {number} index
         * @returns {CertInfo}
         *
         * @memberOf CertInfoCollection
         */
        public items(index: number): CertInfo {
            return new CertInfo(this.handle.items(index));
        }

        /**
         * Return collection length
         *
         * @readonly
         * @type {number}
         * @memberOf CertInfoCollection
         */
        get length(): number {
            return this.handle.length();
        }

    }
}
