/// <reference types="node" />

/* tslint:disable:no-namespace */
/* tslint:disable:no-var-requires */
/* tslint:disable:max-classes-per-file */

declare namespace native {

    namespace PKI {
        class OID {
            constructor(value?: string);
            public getLongName(): string;
            public getShortName(): string;
            public getValue(): string;
        }

        class Extension {
            constructor(oid?: OID, value?: string);
            public getTypeId(): OID;
            public setTypeId(oid: OID): void;
            public getCritical(): boolean;
            public setCritical(critical: boolean): void;
        }

        class ExtensionCollection {
            public items(index: number): Extension;
            public length(): number;
            public push(ext: Extension): void;
            public pop(): void;
            public removeAt(index: number): void;
        }

        class CRL {
            public getVersion(): number;
            public getIssuerName(): string;
            public getIssuerFriendlyName(): string;
            public getLastUpdate(): string;
            public getNextUpdate(): string;
            public getThumbprint(): Buffer;
            public getSignatureAlgorithm(): string;
            public getSignatureDigestAlgorithm(): string;
            public getAuthorityKeyid(): Buffer;
            public getCrlNumber(): number;

            public load(filename: string, dataFormat?: trusted.DataFormat): void;
            public import(raw: Buffer, dataFormat: trusted.DataFormat): void;
            public save(filename: string, dataFormat: trusted.DataFormat): void;
            public export(dataFormat: trusted.DataFormat): Buffer;
            public compare(crl: CRL): number;
            public equals(crl: CRL): boolean;
            public duplicate(): CRL;
            public hash(digestName: string): Buffer;
        }

        class CrlCollection {
            public items(index: number): CRL;
            public length(): number;
            public push(crl: CRL): void;
            public pop(): void;
            public removeAt(index: number): void;
        }

        class Certificate {
            constructor(param?: PKI.Certificate | PKI.CertificationRequest);
            public getSubjectFriendlyName(): string;
            public getSubjectName(): string;
            public getIssuerFriendlyName(): string;
            public getIssuerName(): string;
            public getNotAfter(): string;
            public setNotAfter(offsetSec?: number): void;
            public getNotBefore(): string;
            public setNotBefore(offsetSec?: number): void;
            public getSerialNumber(): Buffer;
            public setSerialNumber(serial: string): void;
            public getThumbprint(): Buffer;
            public getVersion(): number;
            public getType(): number;
            public getKeyUsage(): number;
            public getKeyUsageString(): string[];
            public getEnhancedKeyUsage(): string[];
            public getSignatureAlgorithm(): string;
            public getSignatureDigestAlgorithm(): string;
            public getPublicKeyAlgorithm(): string;
            public getOrganizationName(): string;
            public getOCSPUrls(): string[];
            public getCAIssuersUrls(): string[];
            public getSubjectKeyIdentifier(): Buffer;
            public isSelfSigned(): boolean;
            public isCA(): boolean;

            public sign(): void;
            public load(filename: string, dataFormat?: trusted.DataFormat): void;
            public import(raw: Buffer, dataFormat: trusted.DataFormat): void;
            public save(filename: string, dataFormat: trusted.DataFormat): void;
            public export(dataFormat: trusted.DataFormat): Buffer;
            public compare(cert: Certificate): number;
            public equals(cert: Certificate): boolean;
            public duplicate(): Certificate;
            public hash(digestName: string): Buffer;

            public view(): void;
        }

        class CertificateCollection {
            public items(index: number): Certificate;
            public length(): number;
            public push(cer: Certificate): void;
            public pop(): void;
            public removeAt(index: number): void;
        }

        class Cipher {
            constructor();
            public setProvAlgorithm(name: string): void;
            public encrypt(filenameSource: string, filenameEnc: string, alg?: trusted.EncryptAlg, format?: trusted.DataFormat): void;
            public encryptAsync(filenameSource: string, filenameEnc: string, done: (msg: string) => void, alg?: trusted.EncryptAlg, format?: trusted.DataFormat): void;
            public decrypt(filenameEnc: string, filenameDec: string, format: trusted.DataFormat): void;
            public decryptAsync(filenameEnc: string, filenameDec: string, done: (msg: string) => void, format: trusted.DataFormat): void;
            public addRecipientsCerts(certs: CertificateCollection): void;
            public getRecipientInfos(filenameEnc: string, format: trusted.DataFormat): CMS.CertInfoCollection;
            public setKuPolicy(policyToSet: "kupNone" | "kupDisableKeyAgree" | "kupDisableKeyEnc" | "kupCheckOneOf"): void;
        }

        export interface INameField {
            /**
             * longName | shortName | nid
             *
             * @type {string}
             * @memberof INameField
             */
            type: string;
            value: string;
        }

        class CertificationRequest {
            constructor();
            //public load(filename: string, dataFormat?: trusted.DataFormat): void;
            public save(filename: string, dataFormat?: trusted.DataFormat): void;
            //public getSubject(): string;
            public setSubject(x509name: string | INameField[]): void;
            //public getPublicKey(): Key;
            //public setPublicKey(key: Key): void;
            public getVersion(): number;
            public setVersion(version: number): void;
            //public getExtensions(): ExtensionCollection;
            public setExtensions(exts: ExtensionCollection): void;
            public setContainerName(x509name: string): void;
            public getContainerName(): string;
            public setPubKeyAlgorithm(PubKeyAlgorithm: string): void;
            public getPubKeyAlgorithm(): string;
            public setExportableFlag(ExportableFlag: boolean): void;
            public getExportableFlag(): boolean;
            public setNewKeysetFlag(newKeysetFlag: boolean): void;
            public getNewKeysetFlag(): boolean;
            public toCertificate(notAfter?: number, serial?: string): Certificate;
            //public sign(key: Key, digest?: string): void;
            //public verify(): boolean;
            //public getPEMString(): Buffer;
            //public toCertificate(days: number, key: Key): Certificate;
        }

        class OCSP {
            constructor(inData: Certificate | Buffer, connSettings?: native.UTILS.ConnectionSettings);

            Export(): Buffer;

            Verify(serviceCert?: PKI.Certificate): number;
            VerifyCertificate(cert: PKI.Certificate): number;
            RespStatus(): number;
            SignatureAlgorithmOid(): string;

            Certificates(): CertificateCollection;
            ProducedAt(): string;
            RespNumber(): number;
            RespIndexByCert(cert: PKI.Certificate, issuer?: PKI.Certificate): number;
            OcspCert(): PKI.Certificate;
            getOcspCert(certs?: PKI.CertificateCollection): PKI.Certificate;
            Status(respIdx?: number): number;
            RevTime(respIdx?: number): string;
            RevReason(respIdx?: number): number;
            ThisUpdate(respIdx?: number): string;
            NextUpdate(respIdx?: number): string;
        }

        class TSPRequest {
            constructor(hashAlgOid: string, dataFileName?: string);

            AddData(data: Buffer): void;
            GetCertReq(): boolean;
            SetCertReq(certReq: boolean): void;
            GetNonce(): boolean;
            SetNonce(certReq: boolean): void;
            GetPolicyId(): string;
            SetPolicyId(policyId: string): void;
            GetHashAlgOid(): string;
            SetHashAlgOid(policyId: string): void;
            GetDataHash(): Buffer;
            SetDataHash(dataHash: Buffer): void;
        }

        class TSP {
            constructor(inData: TSPRequest | Buffer, connSettings?: native.UTILS.ConnectionSettings);

            Export(): Buffer;

            Certificates(): CertificateCollection;
            TSACertificate(): Certificate;
            Verify(): number;
            VerifyCertificate(cert: PKI.Certificate): number;
            FailInfo(): number;
            Status(): number;
            StatusString(): string;
            DataHashAlgOID(): string;
            DataHash(): Buffer;
            PolicyID(): string;
            SerialNumber(): Buffer;
            Time(): string;
            Accuracy(): number;
            Ordering(): boolean;
            HasNonce(): boolean;
            TsaName(): string;
            TsaNameBlob(): Buffer;
        }

        class PKCS12 {
            public load(filename: string): void;
            public save(filename: string): void;
        }
    }

    namespace UTILS {
        export interface IContainerName {
            container: string;
            unique: string;
            fqcnA: string;
            fqcnW: string;
        }

        class Csp {
            public isGost2001CSPAvailable(): boolean;
            public isGost2012_256CSPAvailable(): boolean;
            public isGost2012_512CSPAvailable(): boolean;
            public checkCPCSPLicense(): boolean;
            public getCPCSPLicense(): string;
            public getCPCSPVersion(): string;
            public getCPCSPVersionPKZI(): string;
            public getCPCSPVersionSKZI(): string;
            public getCPCSPSecurityLvl(): string;
            public enumProviders(): object[];
            public enumContainers(type?: number, provName?: string): IContainerName[];
            public getCertificateFromContainer(contName: string, provType: number, provName?: string): PKI.Certificate;
            public getContainerNameByCertificate(cert: PKI.Certificate, category: string): string;
            // tslint:disable-next-line:max-line-length
            //public installCertificateFromCloud(cert: PKI.Certificate, authURL: string, restURL: string, certificateID: number): void;
            public installCertificateFromContainer(contName: string, provType: number, provName?: string): void;
            // tslint:disable-next-line:max-line-length
            public installCertificateToContainer(cert: PKI.Certificate, contName: string, provType: number, provName?: string): void;
            public deleteContainer(contName: string, provType: number, provName?: string): void;
            public hasPrivateKey(cert: PKI.Certificate): boolean;
            public buildChain(cert: PKI.Certificate): PKI.CertificateCollection;
            public buildChainAsync(cert: PKI.Certificate, done: (error: string, certs: native.PKI.CertificateCollection) => void): void;
            public verifyCertificateChain(cert: PKI.Certificate): boolean;
            public verifyCertificateChainAsync(cert: PKI.Certificate, done: (error: string, result: boolean) => void): void;
            public verifyCRL(crl: PKI.CRL): boolean;
            public isHaveExportablePrivateKey(cert: PKI.Certificate): boolean;
            public certToPkcs12(cert: PKI.Certificate, exportPrivateKey: boolean, password?: string): PKI.PKCS12;
            public importPkcs12(p12: PKI.PKCS12, password?: string): void;

        }
        class ModuleInfo {
            public getModuleVersion(): string;
            public getModuleName(): string;
            public getCadesEnabled(): boolean;
        }
        class Tools {
            public stringFromBase64(instr: string, flag?: number): string;
            public stringToBase64(instr: string, flag?: number): string;
        }
        class License_Mng {
            public addLicense(lic: string): number;
            public addLicenseFromFile(filename: string): number;
            public deleteLicense(lic: string): boolean;
            public deleteLicenseOfIndex(index: number): boolean;
            public getCountLicense(): number;
	    public getLicense(index: number): string;
	    public checkLicense(lic: string): string;
	    public checkLicenseOfIndex(index: number): string;
	    public generateTrial(): string;
	    public checkTrialLicense(): string;
	    public accessOperations(type:number): boolean;
        }
        class Jwt {
            public createHeader(alg: string): string;
            public createPayload(aud: string, sub: string, core: number, nbf: number, iss: string, exp: number, iat: number, jti: string, desc: string): string;
            public createJWTToken(header: string, payload: string, privateKey: string): string;
            public verifyJWTToken(jwtToken: string, publicKey: string): string;
        }
        class Dlv {
            public licenseValidateFormat(lic: string): boolean;
            public checkLicense(lic: string): string;
        }
        class ConnectionSettings {
            public AuthType: number;
            public Address: string;
            public UserName: string;
            public Password: string;
            public ClientCertificate: PKI.Certificate;
            public ProxyAuthType: number;
            public ProxyAddress: string;
            public ProxyUserName: string;
            public ProxyPassword: string;
        }
        class Hash {
            constructor(hash_alg?: trusted.HashAlg);
            public addData(buffer: Buffer);
            public getValue(): Buffer;
            public hashData(hash_alg: trusted.HashAlg, data: Buffer | string): Buffer;
            public hashDataAsync(hash_alg: trusted.HashAlg, data: Buffer | string, done: (error: string, hashValue: Buffer) => void): Buffer;
        }
    }

    namespace COMMON {
        class Logger {
            public start(filename: string, level: trusted.LoggerLevel): void;
            public stop(): void;
            public clear(): void;
        }
    }

    namespace CMS {
        class TimestampParams {
            //constructor();
            public getStampType(): number;
            public setStampType(stmp: number): void;
            public getConnSettings(): native.UTILS.ConnectionSettings;
            public setConnSettings(connSett: native.UTILS.ConnectionSettings): void;
            public getTspHashAlg(): String;
            public setTspHashAlg(hashAlg: String): void;
        }

        class CadesParams {
            //constructor();
            public getCadesType(): number;
            public setCadesType(signType: number): void;
            public getConnSettings(): native.UTILS.ConnectionSettings;
            public setConnSettings(connSett: native.UTILS.ConnectionSettings): void;
            public getTspHashAlg(): String;
            public setTspHashAlg(hashAlg: String): void;
            public getOcspSettings(): native.UTILS.ConnectionSettings;
            public setOcspSettings(connSett: native.UTILS.ConnectionSettings): void;
        }

        class SignedData {
            constructor();
            public getContent(): Buffer;
            public setContent(v: Buffer): void;
            public getIsContentAvailable(): boolean;
            public writeContent(targetFileName: string, done: (error?: string) => void): void;
            public setContentAsHash(data: Buffer, hash_alg: trusted.HashAlg): void;
            public freeContent(): void;
            public getFlags(): number;
            public setFlags(v: number): void;
            public load(filename: string, dataFormat?: trusted.DataFormat): void;
            public loadAsync(filename: string, dataFormat: trusted.DataFormat, done: (message: string)=>void): void;
            public import(raw: Buffer, dataFormat: trusted.DataFormat): void;
            public importAsync(raw: Buffer, dataFormat: trusted.DataFormat, done: (message: string)=>void): void;
            public save(filename: string, dataFormat: trusted.DataFormat): void;
            public saveAsync(filename: string, dataFormat: trusted.DataFormat, done: (message: string)=>void): void;
            public export(dataFormat?: trusted.DataFormat): Buffer;
            public exportAsync( dataFormat: trusted.DataFormat, done: (message: string, result: Buffer)=>void): void;
            public getCertificates(): PKI.CertificateCollection;
            public getSigners(): SignerCollection;
            public isDetached(): boolean;
            public verify(signer?: CMS.Signer): boolean;
            public verifyAsync(done: (message: string, result: boolean)=>void, signer?: CMS.Signer): void;
            public getSignParams(): CMS.TimestampParams | CMS.CadesParams;
            public setSignParams(params: CMS.TimestampParams | CMS.CadesParams): void;
            public sign(certs: PKI.Certificate): void;
            public signAsync(cert: PKI.Certificate, done: (message: string)=>void): void;
        }

        class SignerCollection {
            public items(index: number): Signer;
            public length(): number;
        }

        class Signer {
            constructor(nativeHandle?: native.CMS.Signer);
            public setCertificate(cert: PKI.Certificate): void;
            public getCertificate(): PKI.Certificate;
            public setIndex(ind: number): void;
            public getIndex(): number;
            public getIssuerName(): string;
            public getSerialNumber(): string;
            // public getSignature(): Buffer;
            public getSignatureAlgorithm(): string;
            public getDigestAlgorithm(): string;
            // public getSignerId(): SignerId;
            // public getSignedAttributes(): SignerAttributeCollection;
            // public getUnsignedAttributes(): SignerAttributeCollection;
            public getSigningTime(): string;
            // public verify(): boolean;
            // public verifyContent(v: Buffer): boolean;
            public timestamp(tspType: number): native.PKI.TSP;
            public verifyTimestamp(tspType: number): boolean;
            public isCades(): boolean;
            public cadesType(): number;
            public certificateValues(): PKI.CertificateCollection;
            public revocationValues(): Buffer[];
            public ocspResp(): native.PKI.OCSP;
            public verifySignatureValue(
                hashValue: Buffer,
                signValue: Buffer,
                signAlg: string | undefined,
                signCertOrKeyValue: PKI.Certificate | Buffer,
                keyAlg?: string,
                namedCurve?: string,
            ): boolean;
        }

        class CertInfo {
            constructor(nativeHandle?: native.CMS.CertInfo);
            public getIssuerName(): string;
            public getSerialNumber(): string;
        }

        class CertInfoCollection {
            public items(index: number): CertInfo;
            public length(): number;
        }
    }

    export namespace PKISTORE {
        export interface IPkiItem extends IPkiCrl, IPkiCertificate, IPkiRequest, IPkiKey {
            /**
             * DER | PEM
             */
            format: string;
            /**
             * CRL | CERTIFICATE | KEY | REQUEST
             */
            type: string;
            uri: string;
            provider: string;
            category: string;
            hash: string;
        }

        export interface IPkiKey {
            encrypted?: boolean;
        }

        export interface IPkiCrl {
            authorityKeyid?: string;
            crlNumber?: string;
            issuerName?: string;
            issuerFriendlyName?: string;
            lastUpdate?: string;
            nextUpdate?: string;
        }

        export interface IPkiRequest {
            subjectName?: string;
            subjectFriendlyName?: string;
            key?: string; // thumbprint ket SHA1
        }

        export interface IPkiCertificate {
            subjectName?: string;
            subjectFriendlyName?: string;
            issuerName?: string;
            issuerFriendlyName?: string;
            notAfter?: string;
            notBefore?: string;
            serial?: string;
            key?: string; // thumbprint ket SHA1
            organizationName?: string;
            signatureAlgorithm?: string;
            signatureDigestAlgorithm?: string;
            publicKeyAlgorithm?: string;
        }

        export interface IFilter {
            /**
             * PkiItem
             * CRL | CERTIFICATE | KEY | REQUEST
             */
            type?: string[];
            /**
             * Provider
             * SYSTEM, MICROSOFT, CRYPTOPRO, TSL, PKCS11, TRUSTEDNET
             */
            provider?: string[];
            /**
             * MY, OTHERS, TRUST, CRL
             */
            category?: string[];
            hash?: string;
            subjectName?: string;
            subjectFriendlyName?: string;
            issuerName?: string;
            issuerFriendlyName?: string;
            isValid?: boolean;
            serial?: string;
        }

        abstract class Provider {
            public type: string;
        }

        class ProviderMicrosoft extends Provider {
            constructor();
        }

        class ProviderCryptopro extends Provider {
            constructor();

            public hasPrivateKey(cert: PKI.Certificate): boolean;
        }

        class PkiStore {
            constructor(json: string);

            public getCash(): CashJson;


            public find(filter?: Filter): IPkiItem[];


            public findKey(filter: Filter): IPkiItem;

            /**
             * Возвращает объект из структуры
             */
            public getItem(item: PkiItem): any;
            public getCerts(): PKI.CertificateCollection;

            public addProvider(provider: Provider): void;

            public addCert(provider: Provider, category: string, cert: PKI.Certificate,
                contName?: string, provType?: number): string;
            public addCrl(provider: Provider, category: string, crl: PKI.CRL): string;

            public deleteCert(provider: Provider, category: string, cert: PKI.Certificate): void;
            public deleteCrl(provider: Provider, category: string, crl: PKI.CRL): void;
            public importSST(sstFileNameOrBuffer: string | Buffer): PKI.CertificateCollection;

        }

        class CashJson {
            public filenName: string;
            constructor(fileName: string);
            public save(fileName: string);
            public load(fileName: string);
            public export(): IPkiItem[];
            public import(items: IPkiItem[] | PkiItem);
        }

        class Filter {
            constructor();
            public setType(type: string): void;
            public setProvider(provider: string): void;
            public setCategory(category: string): void;
            public setHash(hash: string): void;
            public setSubjectName(subjectName: string): void;
            public setSubjectFriendlyName(subjectFriendlyName: string): void;
            public setIssuerName(issuerName: string): void;
            public setIssuerFriendlyName(issuerFriendlyName: string): void;
            public setIsValid(valid: boolean): void;
            public setSerial(serial: string): void;
        }

        class PkiItem {
            constructor();
            public setFormat(type: string): void;
            public setType(type: string): void;
            public setProvider(provider: string): void;
            public setCategory(category: string): void;
            public setURI(category: string): void;
            public setHash(hash: string): void;
            public setSubjectName(subjectName: string): void;
            public setSubjectFriendlyName(subjectFriendlyName: string): void;
            public setIssuerName(issuerName: string): void;
            public setIssuerFriendlyName(issuerFriendlyName: string): void;
            public setSerial(serial: string): void;
            public setNotBefore(before: string): void;
            public setNotAfter(after: string): void;
            public setLastUpdate(lastUpdate: string): void;
            public setNextUpdate(nextUpdate: string): void;
            public setAuthorityKeyid(authorityKeyid: string): void;
            public setCrlNumber(crlNumber: string): void;
            public setKey(key: string): void;
            public setKeyEncrypted(enc: boolean): void;
            public setOrganizationName(organizationName: string): void;
            public setSignatureAlgorithm(signatureAlgorithm: string): void;
            public setSignatureAlgorithm(signatureAlgorithm: string): void;
            public setSignatureDigestAlgorithm(signatureDigestAlgorithm: string): void;
            public setPublicKeyAlgorithm(publicKeyAlgorithm: string): void;
        }
    }
}
