/// <reference path="../native.ts" />
/// <reference path="../object.ts" />

namespace trusted.utils {
    /**
     * JSON Web Token (JWT)
     * Uses only with CTGOSTCP
     *
     * @export
     * @class Jwt
     * @extends {BaseObject<native.JWT.Jwt>}
     */
    export class Jwt extends BaseObject<native.UTILS.Jwt> {
        /**
         * Creates an instance of Jwt.
         *
         *
         * @memberOf Jwt
         */
        constructor() {
            super();
            this.handle = new native.UTILS.Jwt();
        }

        /**
         * Create Header JWT
         * Return 0 if license correct
         *
         * @returns {number}
         *
         * @memberOf Jwt
         */
        public createHeader(alg: string): string {
            return (this.handle.createHeader(alg));
        }

        /**
         * Create Payload JWT
         * Return 0 if license correct
         *
         * @returns {number}
         *
         * @memberOf Jwt
         */
        public createPayload(aud :string , sub :string, core: number, nbf : number, iss : string, exp : number, iat : number, jti : string, desc : string): string {
            return this.handle.createPayload(aud, sub, core, nbf, iss, exp, iat, jti, desc);
        }

        /**
         * Create JWT Token
         *
         * @returns {number}
         *
         * @memberOf Jwt
         */
        public createJWTToken(header : string, payload : string, privateKey : string): string {
            return this.handle.createJWTToken(header, payload, privateKey);
        }

        /**
         * Verify JWT Token
         *
         * @returns {number}
         *
         * @memberOf Jwt
         */
        public verifyJWTToken(jwtToken : string, publicKey : string): string {
            return this.handle.verifyJWTToken(jwtToken, publicKey);
        }
    }
}
