/// <reference path="../native.ts" />
/// <reference path="../object.ts" />

namespace trusted.utils {
    /**
     * ModuleInfo class
     *
     * @export
     * @class ModuleInfo
     * @extends {BaseObject<native.UTILS.ModuleInfo>}
     */
    export class ModuleInfo extends BaseObject<native.UTILS.ModuleInfo> {
        /**
         * Return module version
         *
         * @readonly
         * @type {string}
         * @memberOf ModuleInfo
         */
        get version(): string {
            return this.handle.getModuleVersion();
        }
        /**
         * Return module name
         *
         * @readonly
         * @type {string}
         * @memberOf ModuleInfo
         */
        get name(): string {
            return this.handle.getModuleName();
        }
        /**
         * CAdES support flag
         *
         * @readonly
         * @type {boolean}
         * @memberOf ModuleInfo
         */
        get cadesEnabled(): boolean {
            return this.handle.getCadesEnabled();
        }
        /**
         * Creates an instance of ModuleInfo.
         *
         *
         * @memberOf ModuleInfo
         */
        constructor() {
            super();
            this.handle = new native.UTILS.ModuleInfo();
        }
    }
}
